from enum import StrEnum


class FoundationModels(StrEnum):
    LLAMA_3_8B_INSTRUCT = "meta.llama3-8b-instruct-v1:0"
    LLAMA_3_1_8B_INSTRUCT = "meta.llama3-1-8b-instruct-v1:0"
    LLAMA_3_70B_INSTRUCT = "meta.llama3-70b-instruct-v1:0"
    LLAMA_3_1_70B_INSTRUCT = "meta.llama3-1-70b-instruct-v1:0"
